package operations;

import structures.*;

public class Evaluator implements Visitor {
	private int result;
	
	public Evaluator() {
		this.result = 0;
	}
	
	public int result() {
		return this.result;
	}
	
	public void visitConstant(Constant e) {
		this.result = e.value();
	}
	
	public void visitAddition(Addition e) {
		int[] results = this.visitBinaryOp(e);
		this.result = results[0] + results[1];
	} 
	
	public void visitSubtraction(Subtraction e) {
		int[] results = this.visitBinaryOp(e);
		this.result = results[0] - results[1];
	}
	
	private int[] visitBinaryOp(CompositeExpression ce) {
		Evaluator evalL = new Evaluator();
		Evaluator evalR = new Evaluator();
		ce.getLeft().accept(evalL);
		ce.getRight().accept(evalR);
		int resultL = evalL.result();
		int resultR = evalR.result();
		return new int[] {resultL, resultR};
	}
	
	/*
	 * Less ideal version where code duplicates exist for declaring and calling visitors. 
	 */
//	public void visitAddition(Addition e) {
//		Evaluator evalL = new Evaluator();
//		Evaluator evalR = new Evaluator();
//		e.getLeft().accept(evalL);
//		e.getRight().accept(evalR);
//		this.result = evalL.result() + evalR.result();
//	}
//	
//	public void visitSubtraciton(Subtraction e) {
//		Evaluator evalL = new Evaluator();
//		Evaluator evalR = new Evaluator();
//		e.getLeft().accept(evalL);
//		e.getRight().accept(evalR);
//		this.result = evalL.result() - evalR.result();
//	}
}
