package tests;

import static org.junit.Assert.*;

import org.junit.Test;

import structures.*;
import operations.*;

public class ExpressionTests {
	
	@Test
	public void test_expression_evaluation() {
	  CompositeExpression add; 
	  Expression c1, c2; 
	  Visitor v;
	  c1 = new Constant(1); 
	  c2 = new Constant(2);
	  add = new Addition(c1, c2);
	  v = new Evaluator();
	  add.accept(v);
	  assertEquals(3, ((Evaluator) v).result());
	}

	@Test
	public void test_expression_structure() {
		Expression e2;
		Constant c3, c6;
		Addition add; 
		Subtraction sub;
		
		e2 = new Constant(2); 
		c3 = new Constant(3);
		c6 = new Constant(6);
		
		sub = new Subtraction(e2, c3); // sub: 2 - 3
		add= new Addition(sub, c6); // add: (2 - 3) + 6
		
		/* Study these assertions carefully, as to why they check the structure of the expression. */
		assertTrue(add.getLeft() instanceof Subtraction);
		assertTrue(((Subtraction) add.getLeft()).getLeft() instanceof Constant);
		assertEquals(2, ((Constant) ((Subtraction) add.getLeft()).getLeft()).value());
		assertTrue(((Subtraction) add.getLeft()).getRight() instanceof Constant);
		assertEquals(3, ((Constant) ((Subtraction) add.getLeft()).getRight()).value());
		
		assertTrue(add.getRight() instanceof Constant);
		assertEquals(6, ((Constant) add.getRight()).value());
	}

	@Test
	public void test_expression_operations() {
		Expression e2;
		Constant c3, c6;
		Addition add; 
		Subtraction sub;
		
		e2 = new Constant(2); 
		c3 = new Constant(3);
		c6 = new Constant(6);
		
		sub = new Subtraction(e2, c3); // sub: 2 - 3
		add= new Addition(sub, c6); // add: (2 - 3) + 6
		
		/* Apply the relevant visitor for pretty printing. */
		PrettyPrinter pp = new PrettyPrinter();
		add.accept(pp);
		assertEquals("((2) - (3)) + (6)", pp.result());
		
		/* Apply the relevant visitor for evaluation. */
		Evaluator eval = new Evaluator();
		add.accept(eval);
		assertEquals((2 - 3) + 6, eval.result());
	}
}
